#!/usr/bin/env python
# -*- coding: utf8 -*-

# PlumHound https://github.com/PlumHound/PlumHound | https://plumhound.defensiveorigins.com/
# BloodHoundAD Wrapper for Purple Teams | https://github.com/BloodHoundAD/BloodHound
# BloodHound CE (Newest version) can also be used | https://github.com/SpecterOps/BloodHound

# GNU GPL 3.0

import sys
import traceback

# Import PlumHound libraries
import lib.phCheckPython 
import lib.phCLImanagement
import lib.phNotifyArgs
import lib.phTasks
import lib.phDatabase

ph_version = "1.6"


def main() -> int:
    """
    Main entry point for PlumHound CLI. Returns an exit code (0 success, non-zero error).
    This function is referenced by the pyproject entry point: PlumHound:main
    """
    try:
        # Check if Py2 environment.  If not executing in Python3, exit nicely.
        lib.phCheckPython.CheckPython2()

        # Commandline Arguments (ArgParse) configuration
        phArgs = lib.phCLImanagement.SetupArguments(ph_version)

        # Check if Py3 environment.  If not expected version of 3, notify nicely
        lib.phCheckPython.CheckPython3(phArgs)

        # Report execution parameters
        lib.phNotifyArgs.NotifyArgs(phArgs, ph_version)

        # Generate TaskList (jobs)
        phTaskList = lib.phTasks.MakeTaskList(phArgs)

        # Setup Driver (excluding BlueHound)
        phDriver = lib.phDatabase.setup_database_conn(phArgs)

        # Execute Jobs in Task List
        lib.phTasks.TaskExecution(phTaskList, phDriver, phArgs)

        # Close the neoj4 connection.
        lib.phDatabase.close_database_con(phArgs, phDriver)

        return 0
    except SystemExit:
        # Let argparse or other code call sys.exit() behave normally
        raise
    except Exception:
        # Print traceback for visibility when used as a console command
        traceback.print_exc()
        return 1


if __name__ == "__main__":
    sys.exit(main())




#!/usr/bin/env python
# -*- coding: utf8 -*-
# PlumHound (phDatabase.py) Database Connection Management
# https://github.com/PlumHound/PlumHound 
# License GNU GPL3

from neo4j import GraphDatabase
from lib.phLoggy import Loggy as Loggy

# Setup Database Connection
def setup_database_conn(phArgs):
    Loggy(phArgs.verbose,900, "------ENTER: SETUP_DATABASE_CONN-----")
    Loggy(phArgs.verbose,200, "[!] Attempting to connect to your Neo4j project using {}:{} @ {} {}.".format(phArgs.username, "****", phArgs.server, "[ENCRYPTED]" if phArgs.UseEnc else "[UNECNCRYPTED]"))
    try:
        if phArgs.UseEnc:
            Loggy(phArgs.verbose,200, " Using Neo4j encryption")
            driver_connection = GraphDatabase.driver(phArgs.server, auth=(phArgs.username, phArgs.password), encrypted=True)
        else:
            Loggy(phArgs.verbose,200, " Not using Neo4j encryption")
            driver_connection = GraphDatabase.driver(phArgs.server, auth=(phArgs.username, phArgs.password), encrypted=False)
        Loggy(phArgs.verbose,200, "[+] Success!")
        return driver_connection
    except Exception:
        Loggy(phArgs.verbose,100, "There was a problem. Check username, password, and server parameters.")
        Loggy(phArgs.verbose,100, "[X] Database connection failed!")
        exit()
    Loggy(phArgs.verbose,900, "------EXIT: SETUP_DATABASE_CONN-----")

def close_database_con(phArgs,connectiondriver):
    Loggy(phArgs.verbose,900, "------ENTER: CLOSE_DATABASE_CONN-----")
    connectiondriver.close
    Loggy(phArgs.verbose,200, "[+] Closed Database Connection!")
    Loggy(phArgs.verbose,900, "------EXIT: CLOSE_DATABASE_CONN-----")
    exit()



#!/usr/bin/env python
# -*- coding: utf8 -*-
# PlumHound (phReportIndexer.py) - Build Index of Reports (jobs)
# https://github.com/PlumHound/PlumHound 
# License GNU GPL3

#Python Libraries
from tabulate import tabulate
from datetime import date

#Plumhound Modules
from lib.phLoggy import Loggy as Loggy

def ReportIndexer(verbose,Processed_Results_List, OutPathFile, HTMLHeader, HTMLFooter, HTMLCSS):
    Loggy(verbose,900, "------ENTER: REPORT INDEXER-----")

    list_KeysList = ["Title", "Count", "Further Details"]
    Title = "Full Report Details"

    Loggy(verbose,200, "Beginning Output HTML:" + OutPathFile)

    for entry in Processed_Results_List:
        if entry[3] == "HTML":
            filename = entry[2]
            entry[2] = "<a href=\"" + filename + "\">Details</a>"
        elif entry[3] == "CSV":
            filename = entry[2]
            entry[2] = "<a href=\"" + filename + "\">CSV</a>"
        elif entry[3] == "HTMLCSV":
            filename = entry[2]
            entry[2] = "<a href=\"" + filename + ".html\">Details</a> - <a href=\"" + filename + ".csv\">CSV</a> "
        #Remove the unnecessary 4th item to correct the keys 
        del entry[3]

    output = str(tabulate(Processed_Results_List, list_KeysList, tablefmt="html"))
    output = output.replace("&lt;","<")
    output = output.replace("&gt;",">")
    output = output.replace("&quot;",'"')

    HTMLCSS_str = ""
    HTMLHeader_str = ""
    HTMLFooter_str = ""
    HTMLPre_str = "<HTML><head>"
    HTMLMId_str = "</head><Body>"
    HTMLEnd_str = "</body></html>"
    if HTMLHeader:
        with open(HTMLHeader, 'r') as header:
            HTMLHeader_str = header.read()
        HTMLHeader_str = ReplaceHTMLReportVars(HTMLHeader_str, Title)

    if HTMLFooter:
        with open(HTMLFooter, 'r') as footer:
            HTMLFooter_str = footer.read()
        HTMLFooter_str = ReplaceHTMLReportVars(HTMLFooter_str, Title)

    if HTMLCSS:
        with open(HTMLCSS, 'r') as css:
            HTMLCSS_str = "<style>\n" + css.read() + "\n</style>"

    Loggy(verbose,500, "File Writing " + OutPathFile)
    output = HTMLPre_str + HTMLCSS_str + HTMLMId_str + HTMLHeader_str + output + HTMLFooter_str + HTMLEnd_str
    fsys = open(OutPathFile, "w")
    fsys.write(output)
    fsys.close
    Loggy(verbose,110, "Full index report written to " + OutPathFile)
    return True
    Loggy(verbose,900, "------EXIT: REPORT INDEXER-----")


def ReplaceHTMLReportVars(InputStr, Title):
    sOutPut = InputStr.replace("--------PH_TITLE-------", str(Title))
    sOutPut = sOutPut.replace("--------PH_DATE-------", str(date.today()))
    return sOutPut



#!/usr/bin/env python
# -*- coding: utf8 -*-
# PlumHound - Output Delivery and Parsing Functions(phdeliver.py)
# https://github.com/PlumHound/PlumHound 
# License GNU GPL3

#Python Libraries
import ast
import csv
from tabulate import tabulate
from datetime import datetime

#Plumhound Modules
from lib.phLoggy import Loggy as Loggy

def SenditOut(verbose,list_KeysList, Processed_Results_List, OutFormat, OutFile, OutPath, Title, HTMLHeader, HTMLFooter, HTMLCSS, jobQuery):
    Loggy(verbose,900, "------ENTER: SENDITOUT-----")
    # Quick fix if keys returned no records to properly rebuild the keys list of 0, instead of int(0)
    if isinstance(list_KeysList, int):
        list_KeysList = []
    output = ""

    if OutFormat == "CSV":
        Loggy(verbose,500, "Beginning Output CSV:" + OutPath + OutFile)
        with open(OutPath + OutFile, "w", newline="", encoding="utf-8") as f:
            Loggy(verbose,500, "KeyType: " + str(type(list_KeysList)))
            Loggy(verbose,500, "KeyList: " + str((list_KeysList)))
            writer = csv.writer(f)
            ModKeyList = ast.literal_eval("[" + str(list_KeysList) + "]")
            Loggy(verbose,500, "KeyTypeMod: " + str(type(ModKeyList)))
            Loggy(verbose,500, "KeyListMod: " + str(ModKeyList))
            writer.writerows(ModKeyList)
            Loggy(verbose,500, "ResultsType: " + str(type(Processed_Results_List)))
            Loggy(verbose,999, "ResultsList: " + str(Processed_Results_List))
            writer.writerows(Processed_Results_List)
        Loggy(verbose,150, "Task " + Title + " Complete: " + OutFile)
        return True

    if OutFormat == "HTMLCSV":
        Loggy(verbose,500, "Beginning Output HTMLCSV:" + OutPath + OutFile)
        SenditOut(verbose,list_KeysList, Processed_Results_List, "HTML", OutFile + ".html", OutPath, Title, HTMLHeader, HTMLFooter, HTMLCSS, jobQuery)
        SenditOut(verbose,list_KeysList, Processed_Results_List, "CSV", OutFile + ".csv", OutPath, Title, HTMLHeader, HTMLFooter, HTMLCSS, jobQuery)
        Loggy(verbose,500, "Completed Output HTMLCSV:" + OutPath + OutFile)
        return True

    if OutFormat == "STDOUT":
        Loggy(verbose,500, "Beginning Standard Output:")
        print()
        output = tabulate(Processed_Results_List, list_KeysList, tablefmt="simple")
        print(output)
        print()
        return True

    if OutFormat == "HTML":
        Loggy(verbose,500, "Beginning Output HTML:" + OutFile)
        output = tabulate(Processed_Results_List, list_KeysList, tablefmt="html")
        outputq = "<br><div style='width:50%; display:inline-block;'><tt><u>Cypher Query</u>:<br>"+jobQuery+"</tt></div>"
        HTMLCSS_str = ""
        HTMLHeader_str = ""
        HTMLFooter_str = ""
        HTMLPre_str = "<HTML><head>"
        HTMLMId_str = "</head><Body>"
        HTMLEnd_str = "</body></html>"
        if HTMLHeader:
            with open(HTMLHeader, 'r') as header:
                HTMLHeader_str = header.read()
            HTMLHeader_str = ReplaceHTMLReportVars(HTMLHeader_str, Title)

        if HTMLFooter:
            with open(HTMLFooter, 'r') as footer:
                HTMLFooter_str = footer.read()
            HTMLFooter_str = ReplaceHTMLReportVars(HTMLFooter_str, Title)

        if HTMLCSS:
            with open(HTMLCSS, 'r') as css:
                HTMLCSS_str = "<style>\n" + css.read() + "\n</style>"

        Loggy(verbose,500, "File Writing " + OutPath + OutFile)
        output = HTMLPre_str + HTMLCSS_str + HTMLMId_str + HTMLHeader_str + output + outputq + HTMLFooter_str + HTMLEnd_str
        fsys = open(OutPath + OutFile, "w")
        fsys.write(output)
        Loggy(verbose,150, "Task " + Title + " Complete: " + OutFile)
        Loggy(verbose,500, "File Closing " + OutPath + OutFile)
        fsys.close
        return True
    Loggy(verbose,900, "------EXIT: SENDITOUT-----")

def ReplaceHTMLReportVars(InputStr, Title):
    now=datetime.now()
    sOutPut = InputStr.replace("--------PH_TITLE-------", str(Title))
    sOutPut = sOutPut.replace("--------PH_DATE-------", str(now.strftime("%Y-%m-%d %H:%M:%S")))
    return sOutPut



#!/usr/bin/env python
# -*- coding: utf8 -*-
# PlumHound (phLoggy.py) - Logging Function
# https://github.com/PlumHound/PlumHound 
# License GNU GPL3

# Loggy Function for lazy debugging

import datetime

def Loggy(hurdle, level, notice):
    logfile="log/PlumHound.log"
    if level <= hurdle:
        if level <100: 
            print("" + "\t" + notice)
        elif level == 100:
            print("INFO" + "\t" + notice)
        else:
            print(str(level) + "\t" + notice)

    fsys = open(logfile, "a", encoding="utf-8")
    fsys.writelines(str(level) + "\t" + datetime.datetime.now().strftime("%d.%b %Y %H:%M:%S") +"\t" + notice +"\n")
    fsys.close



#!/usr/bin/env python
# -*- coding: utf8 -*-
# PlumHound (phtasks.py) - Task Management and Execution
# https://github.com/PlumHound/PlumHound 
# License GNU GPL3

#Python Libraries
import ast
import copy
from alive_progress import alive_bar
from neo4j import GraphDatabase
from neo4j import unit_of_work

#Plumhound modules
from lib.phLoggy import Loggy as Loggy
import lib.phDeliver

#Plumhound Extensions
import modules.BlueHound
import modules.AzHound
import modules.ph_ReportIndexer
import modules.ph_TaskZipper

def MakeTaskList(phArgs):
    Loggy(phArgs.verbose,900, "------ENTER: MAKETASKLIST-----")
    Loggy(phArgs.verbose,200, "Building Task List")

    tasks = []

    if phArgs.TaskFile:
        Loggy(phArgs.verbose,500, "Tasks file specified.  Reading")
        with open(phArgs.TaskFile, encoding="utf-8") as f:
            tasks = f.read().splitlines()
        Loggy(phArgs.verbose,500, "TASKS: " + str(tasks))
        Loggy(phArgs.verbose,90, "Found " +str(len(tasks))+" task(s)")
        Loggy(phArgs.verbose,90, "--------------------------------------")
        Loggy(phArgs.verbose,90, "")
        Loggy(phArgs.verbose,90, "")
        return tasks

    if phArgs.QuerySingle:
        Loggy(phArgs.verbose,500, "Tasks Single Query Specified. Reading")
        Loggy(phArgs.verbose,500, "Tasks-Title:" + phArgs.title)
        Loggy(phArgs.verbose,500, "Tasks-OutFormat:" + phArgs.OutFormat)
        Loggy(phArgs.verbose,500, "Tasks-OutPath:" + phArgs.path)
        Loggy(phArgs.verbose,500, "Tasks-QuerySingle:" + phArgs.QuerySingle)

        task_str = "[\"" + phArgs.title + "\",\"" + phArgs.OutFormat + "\",\"" + phArgs.OutFile + "\",\"" + phArgs.QuerySingle + "\"]"
        Loggy(phArgs.verbose,500, "Task_str:  " + task_str)
        tasks = [task_str]

        return tasks

    if phArgs.BusiestPath:
        # Find and print on screen the X Attack Paths that give the most users a path to DA
        bp=modules.BlueHound.find_busiest_path(phArgs.server, phArgs.username, phArgs.password, phArgs.BusiestPath[0], phArgs.BusiestPath[1])

    if phArgs.AnalyzePath:
        if phArgs.AnalyzePath[0].upper() == "USER":
            snode="User"
            enode=""
        elif phArgs.AnalyzePath[0].upper() == "GROUP":
            snode="Group"
            enode=""
        elif phArgs.AnalyzePath[0].upper() == "COMPUTER":
            snode="Computer"
            enode=""
        else:
            snode=(phArgs.AnalyzePath[0]).upper()
            enode=(phArgs.AnalyzePath[1]).upper()
        modules.BlueHound.getpaths(phArgs.server, phArgs.username, phArgs.password,snode,enode)

    if phArgs.AzPath:
        if phArgs.AzPath[0].upper() == "ALL":
            snode="AZUser"
            enode="GLOBAL ADMINISTRATOR"
            modules.AzHound.getpaths(phArgs.server, phArgs.username, phArgs.password,snode,enode)
            snode="AZGroup"
            modules.AzHound.getpaths(phArgs.server, phArgs.username, phArgs.password,snode,enode)
            snode="AZApp"
            modules.AzHound.getpaths(phArgs.server, phArgs.username, phArgs.password,snode,enode)
            snode="AZServicePrincipal"
            modules.AzHound.getpaths(phArgs.server, phArgs.username, phArgs.password,snode,enode)
            snode="AZDevice"
            modules.AzHound.getpaths(phArgs.server, phArgs.username, phArgs.password,snode,enode)
        elif phArgs.AzPath[0].upper() == "AZUSER":
            snode="AZUser"
            enode="GLOBAL ADMINISTRATOR"
        elif phArgs.AzPath[0].upper() == "AZGROUP":
            snode="AZGroup"
            enode="GLOBAL ADMINISTRATOR"
        elif phArgs.AzPath[0].upper() == "AZAPP":
            snode="AZApp"
            enode="GLOBAL ADMINISTRATOR"
        elif phArgs.AzPath[0].upper() == "AZSERVICEPRINCIPAL":
            snode="AZServicePrincipal"
            enode="GLOBAL ADMINISTRATOR"
        elif phArgs.AzPath[0].upper() == "AZDEVICE":
            snode="AZDevice"
            enode="GLOBAL ADMINISTRATOR"           
        else:
            snode=(phArgs.AzPath[0]).upper()
            enode=(phArgs.AzPath[1]).upper()
        if phArgs.AzPath[0].upper() != "ALL":
            modules.AzHound.getpaths(phArgs.server, phArgs.username, phArgs.password,snode,enode)

    if phArgs.easy:
        Loggy(phArgs.verbose,500, "Tasks Easy Query Specified.")
        tasks = ['["Domain Users (Limit 10)","STDOUT","","MATCH (n:User) RETURN n.name, n.displayname LIMIT 10"]']
        Loggy(phArgs.verbose,100, "Found " +str(len(tasks))+" task(s)")
        Loggy(phArgs.verbose,100, "--------------------------------------")
        return tasks

    Loggy(phArgs.verbose,100, "Tasks Generation Completed\nTasks: " + str(tasks))
    Loggy(phArgs.verbose,900, "------EXIT: MAKETASKLIST-----")
    return tasks


# Execute Tasks
def TaskExecution(tasks, phDriver, phArgs):

    Loggy(phArgs.verbose,900, "------ENTER: TASKEXECUTION-----")
    Loggy(phArgs.verbose,500, "Begin Task Executions")
    Loggy(phArgs.verbose,500, "TASKS:" + str(tasks))
    
    Outpath=phArgs.path

    # Correct Path if sent as command line/not default. 
    if "//" not in Outpath:
        Outpath=phArgs.path + "//"

    jobHTMLHeader = phArgs.HTMLHeader
    jobHTMLFooter = phArgs.HTMLFooter
    jobHTMLCSS = phArgs.HTMLCSS

    task_output_list = []

    tasksuccess=0
         
    with alive_bar(len(tasks),title='\t Executing Tasks',length=50,theme='smooth',spinner=None,dual_line=True,monitor="Tasks {count} / {total} ") as tpbar:
        for job in tasks:
            tpbar()
            try:
                Loggy(phArgs.verbose,500, "Job: " + str(job))

                job_List = ast.literal_eval(job)
                jobTitle = job_List[0]
                jobOutFormat = job_List[1]
                jobOutPathFile = Outpath + job_List[2]
                jobQuery = job_List[3]

                tpbar.text("\t -> Task: " + jobTitle)

                Loggy(phArgs.verbose,200, "Starting job: " + jobTitle)

                Loggy(phArgs.verbose,500, "Job Title: " + jobTitle)
                Loggy(phArgs.verbose,500, "Job Format: " + jobOutFormat)
                Loggy(phArgs.verbose,500, "Job File: " + jobOutPathFile)
                Loggy(phArgs.verbose,500, "Job Query: " + jobQuery)

                if jobQuery == "REPORT-INDEX":
                    task_output_list_clean = copy.deepcopy(task_output_list) #copy the list and revert it after ReportIndexer
                    modules.ph_ReportIndexer.ReportIndexer(phArgs.verbose,task_output_list, jobOutPathFile, jobHTMLHeader, jobHTMLFooter, jobHTMLCSS)
                    task_output_list = task_output_list_clean 
                    task_output_list.append([jobTitle, len(jobresults_processed_list), job_List[2],jobOutFormat])
                    tasksuccess += 1
                    continue

                if jobQuery == "ZIP-TASKS":
                    Loggy(phArgs.verbose,200, "task_output_list: " + str(task_output_list))
                    modules.ph_TaskZipper.ZipTasks(phArgs.verbose,task_output_list, jobOutPathFile, Outpath)
                    tasksuccess += 1
                    continue
                
                #now returns native dict containing keys
                jobkeys = get_keys(phArgs.verbose,phDriver, jobQuery)
                jobkeys_List = ast.literal_eval(str(jobkeys))

                # If keys returned 0, make an empty list
                if isinstance(jobkeys_List, int):
                    jobkeys_List = []

                #
                jobresults = execute_query(phArgs.verbose,phDriver, jobQuery)
                jobresults_processed = "[" + process_results(phArgs.verbose,jobresults) + "]"
                try:
                    jobresults_processed_list = ast.literal_eval(jobresults_processed)
                except Exception as er1:
                    Loggy(phArgs.verbose,100, "ERROR While parsing results (non-fatal but errors may exist in output.")
                    print(er1)
                    Loggy(phArgs.verbose,500, jobresults_processed)
                    jobresults_processed_list = jobresults_processed

                if jobOutFormat == "HTML" or jobOutFormat == "HTMLCSV" or jobOutFormat == "CSV":
                    task_output_list.append([jobTitle, len(jobresults_processed_list), job_List[2],jobOutFormat])

                Loggy(phArgs.verbose,500, "Exporting Job Results")
                lib.phDeliver.SenditOut(phArgs.verbose,jobkeys_List, jobresults_processed_list, jobOutFormat, jobOutPathFile, "", jobTitle, jobHTMLHeader, jobHTMLFooter, jobHTMLCSS, jobQuery)
                tasksuccess += 1

            except Exception as er2:
                Loggy(phArgs.verbose,100, "ERROR While running job (trying next job in list).")
                print(er2)

    Loggy(phArgs.verbose,90, "")
    Loggy(phArgs.verbose,90, "Completed " + str(tasksuccess) + " of " + str(len(tasks)) + " tasks.")        
    Loggy(phArgs.verbose,90, "")   


    Loggy(phArgs.verbose,900, "------EXIT: TASKEXECUTION-----")

    if tasksuccess != (len(tasks)):
        Loggy(phArgs.verbose,100, "Completed " + str(tasksuccess) + " of " + str(len(tasks)) + " tasks (Non-Lethal errors occurred).")

    Loggy(phArgs.verbose,90, "")   

# Setup Query
@unit_of_work(timeout=300)
def execute_query(verbose, phDriver, query, enabled=True):
    Loggy(verbose,900, "------ENTER: EXECUTE_QUERY-----")
    try:
        with phDriver.session() as session:
            Loggy(verbose,500, "Running Query")
            results = session.run(query)
            #results return a <class 'neo4j._sync.work.result.Result'>
            #https://neo4j.com/docs/api/python-driver/current/api.html#result
            #I have tried getting the number of records in the object, but its a bit tricky, all functions doing it in one step ive tired have been deprecated and removed in the new driver
            #even tried making a summary object before getting the real data:
            #https://neo4j.com/docs/api/python-driver/current/api.html#neo4j.ResultSummary
            #results = session.run(query)
            #summary = results.consume()
            #results = session.run(query)
            #count = summary.counters.nodes_created()
            #data = results.fetch(count)
            #Bloodhounds javascript is using recursion, id say its the way to go
            data = results.fetch(999999999999)
            if check_records(verbose, phDriver, query):
                Loggy(verbose,500, "Identified " + str(len(data)) + " Results")
            else:
                Loggy(verbose,200, "Job result: No records found")
        Loggy(verbose,900, "------EXIT: EXECUTE_QUERY-----")
    except Exception as er3:
        Loggy(verbose,200,"Error occured in execute_query: ")
        print(er3)
    return data


# Grab Keys for Cypher Query
@unit_of_work(timeout=300)
def get_keys(verbose,phDriver, query, enabled=True):
    Loggy(verbose,900, "------ENTER: get_keys-----")
    try:
        Loggy(verbose,500, "get_keys Query: " + str(query))
        with phDriver.session() as session:
            results = session.run(query)
            keys = results.keys()
            if check_records(verbose, phDriver, query):
                keys = results.keys()
                Loggy(verbose,500, "Identified " + str(len(keys)) + " Results")
                Loggy(verbose,500, "Keys Found")
            else:
                Loggy(verbose,200, "No Keys found")
                keys = 0
        Loggy(verbose,500, "Key enumeration complete")
        Loggy(verbose,900, "------EXIT: get_keys-----")
    except Exception as er4:
        Loggy(verbose,200,"Error occured in get_keys: ")
        print(er4)
    return keys


def check_records(verbose, phDriver, query):
    Loggy(verbose,900, "------ENTER: check_records-----")
    try:
        with phDriver.session() as session:
            results = session.run(query)
            first = results.peek()
            if first:
                Loggy(verbose,500, "Found Records")
            else:
                Loggy(verbose,200, "No Records Found")
            Loggy(verbose,900, "------EXIT: check_records-----")
    except Exception as er5:
        Loggy(verbose,200,"Error occured in check_records: ")
        print(er5)
    return first


def process_results(verbose,results):
    Loggy(verbose,900, "------ENTER: process_results-----")
    BigTable = ""
    for record in results:
        try:
            BigTable = BigTable + str(record.values()) + ","
        except Exception as er6:
            Loggy(verbose,200, "Washing records failed. Error on record")
            print(er6)
    Loggy(verbose,900, "------EXIT: process_results-----")
    return BigTable



#!/usr/bin/env python
# -*- coding: utf8 -*-
# PlumHound (phNotifyArgs.py) - Check and Notify of CL arguments
# https://github.com/PlumHound/PlumHound 
# License GNU GPL3

from lib.phLoggy import Loggy as Loggy

def NotifyArgs(phArgs, ph_version):
    Loggy(phArgs.verbose,10, "")
    Loggy(phArgs.verbose,10, "PlumHound " +str(ph_version))
    Loggy(phArgs.verbose,10, "For more information: https://github.com/plumhound")
    Loggy(phArgs.verbose,50, "--------------------------------------")
    Loggy(phArgs.verbose,50, "Server: " + str(phArgs.server))
    Loggy(phArgs.verbose,50, "User: " + str(phArgs.username))
    Loggy(phArgs.verbose,50, "Password: *****")
    if phArgs.UseEnc:
        Loggy(phArgs.verbose,50, "Encryption: True")
    else:
        Loggy(phArgs.verbose,50, "Encryption: False")
    Loggy(phArgs.verbose,50, "Timeout: " + str(phArgs.timeout))
    
    Loggy(phArgs.verbose,50, "--------------------------------------")
    
    if phArgs.TaskFile:
        Loggy(phArgs.verbose,50, "Tasks: Task File")
        Loggy(phArgs.verbose,50, "TaskFile: " + str(phArgs.TaskFile))
    
    if phArgs.QuerySingle:
        Loggy(phArgs.verbose,50, "Task: Single Query")
        Loggy(phArgs.verbose,50, "Query Title: " + str(phArgs.title))
        Loggy(phArgs.verbose,50, "Query Format: " + str(phArgs.OutFormat))
        Loggy(phArgs.verbose,50, "Query Path: " + str(phArgs.path))
        Loggy(phArgs.verbose,50, "Query Cypher: " + str(phArgs.QuerySingle))
    
    if phArgs.BusiestPath:
        Loggy(phArgs.verbose,50, "Task: Busiest Path")
        Loggy(phArgs.verbose,50, "Argument 1: " + str(phArgs.BusiestPath[0]))
        Loggy(phArgs.verbose,50, "Argument 2: " + str(phArgs.BusiestPath[1]))
    
    if phArgs.AnalyzePath:
        Loggy(phArgs.verbose,50, "Task: Analyzer Path")
        Loggy(phArgs.verbose,50, "Start Node: " + str(phArgs.AnalyzePath[0].upper()))
    
    if phArgs.easy:
        Loggy(phArgs.verbose,50, "Task: Easy")
        Loggy(phArgs.verbose,50, "Query Title: Domain Users")
        Loggy(phArgs.verbose,50, "Query Format: STDOUT")
        Loggy(phArgs.verbose,50, "Query Cypher: MATCH (n:User) RETURN n.name, n.displayname")
        
        Loggy(phArgs.verbose,50, "--------------------------------------")

    return True
